@@ -769,8 +769,8 @@ int g_dhcp_server_start(GDHCPServer *dhcp_server)
 
 	g_io_channel_set_close_on_unref(listener_channel, TRUE);
 	dhcp_server->listener_watch =
-			g_io_add_watch_full(listener_channel,
-						G_PRIORITY_HIGH, G_IO_IN,
+			g_io_add_watch_full(listener_channel, G_PRIORITY_HIGH,
+				G_IO_IN | G_IO_NVAL | G_IO_ERR | G_IO_HUP,
 						listener_event, dhcp_server,
 								NULL);
 	g_io_channel_unref(dhcp_server->listener_channel);
