@@ -101,6 +101,8 @@ struct _GWeb {
 
 	guint next_query_id;
 
+	int family;
+
 	int index;
 	GList *session_list;
 
@@ -203,6 +205,8 @@ GWeb *g_web_new(int index)
 
 	web->next_query_id = 1;
 
+	web->family = AF_UNSPEC;
+
 	web->index = index;
 	web->session_list = NULL;
 
@@ -280,6 +284,21 @@ gboolean g_web_set_proxy(GWeb *web, const char *proxy)
 	return TRUE;
 }
 
+gboolean g_web_set_address_family(GWeb *web, int family)
+{
+	if (web == NULL)
+		return FALSE;
+
+	if (family != AF_UNSPEC && family != AF_INET && family != AF_INET6)
+		return FALSE;
+
+	web->family = family;
+
+	g_resolv_set_address_family(web->resolv, family);
+
+	return TRUE;
+}
+
 gboolean g_web_add_nameserver(GWeb *web, const char *address)
 {
 	if (web == NULL)
@@ -429,13 +448,11 @@ static gboolean process_send_buffer(struct web_session *session)
 		return FALSE;
 	}
 
-	debug(session->web, "bytes to write %zu", count);
-
 	status = g_io_channel_write_chars(session->transport_channel,
 					buf->str, count, &bytes_written, NULL);
 
-	debug(session->web, "status %u bytes written %zu",
-					status, bytes_written);
+	debug(session->web, "status %u bytes to write %zu bytes written %zu",
+					status, count, bytes_written);
 
 	if (status != G_IO_STATUS_NORMAL && status != G_IO_STATUS_AGAIN)
 		return FALSE;
@@ -1061,6 +1078,7 @@ static void resolv_result(GResolvResultStatus status,
 
 	memset(&hints, 0, sizeof(struct addrinfo));
 	hints.ai_flags = AI_NUMERICHOST;
+	hints.ai_family = session->web->family;
 
 	if (session->addr != NULL) {
 		freeaddrinfo(session->addr);
@@ -1157,6 +1175,7 @@ static guint do_request(GWeb *web, const char *url,
 
 		memset(&hints, 0, sizeof(struct addrinfo));
 		hints.ai_flags = AI_NUMERICHOST;
+		hints.ai_family = session->web->family;
 
 		if (session->addr != NULL) {
 			freeaddrinfo(session->addr);
