@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -42,16 +42,9 @@ enum connman_service_type {
 	CONNMAN_SERVICE_TYPE_WIMAX     = 4,
 	CONNMAN_SERVICE_TYPE_BLUETOOTH = 5,
 	CONNMAN_SERVICE_TYPE_CELLULAR  = 6,
-	CONNMAN_SERVICE_TYPE_VPN       = 7,
-};
-
-enum connman_service_mode {
-	CONNMAN_SERVICE_MODE_UNKNOWN = 0,
-	CONNMAN_SERVICE_MODE_MANAGED = 1,
-	CONNMAN_SERVICE_MODE_ADHOC   = 2,
-	CONNMAN_SERVICE_MODE_GPRS    = 3,
-	CONNMAN_SERVICE_MODE_EDGE    = 4,
-	CONNMAN_SERVICE_MODE_UMTS    = 5,
+	CONNMAN_SERVICE_TYPE_GPS       = 7,
+	CONNMAN_SERVICE_TYPE_VPN       = 8,
+	CONNMAN_SERVICE_TYPE_GADGET    = 9,
 };
 
 enum connman_service_security {
@@ -70,6 +63,7 @@ enum connman_service_state {
 	CONNMAN_SERVICE_STATE_ASSOCIATION   = 2,
 	CONNMAN_SERVICE_STATE_CONFIGURATION = 3,
 	CONNMAN_SERVICE_STATE_READY         = 4,
+	CONNMAN_SERVICE_STATE_ONLINE        = 5,
 	CONNMAN_SERVICE_STATE_DISCONNECT    = 6,
 	CONNMAN_SERVICE_STATE_FAILURE       = 7,
 };
@@ -80,6 +74,16 @@ enum connman_service_error {
 	CONNMAN_SERVICE_ERROR_PIN_MISSING    = 2,
 	CONNMAN_SERVICE_ERROR_DHCP_FAILED    = 3,
 	CONNMAN_SERVICE_ERROR_CONNECT_FAILED = 4,
+	CONNMAN_SERVICE_ERROR_LOGIN_FAILED  = 5,
+	CONNMAN_SERVICE_ERROR_AUTH_FAILED    = 6,
+	CONNMAN_SERVICE_ERROR_INVALID_KEY    = 7,
+};
+
+enum connman_service_proxy_method {
+	CONNMAN_SERVICE_PROXY_METHOD_UNKNOWN     = 0,
+	CONNMAN_SERVICE_PROXY_METHOD_DIRECT      = 1,
+	CONNMAN_SERVICE_PROXY_METHOD_MANUAL      = 2,
+	CONNMAN_SERVICE_PROXY_METHOD_AUTO        = 3,
 };
 
 struct connman_service;
@@ -91,8 +95,14 @@ void connman_service_unref(struct connman_service *service);
 enum connman_service_type connman_service_get_type(struct connman_service *service);
 char *connman_service_get_interface(struct connman_service *service);
 
-int connman_service_set_favorite(struct connman_service *service,
-						connman_bool_t favorite);
+const char *connman_service_get_domainname(struct connman_service *service);
+char **connman_service_get_nameservers(struct connman_service *service);
+void connman_service_set_proxy_method(struct connman_service *service, enum connman_service_proxy_method method);
+enum connman_service_proxy_method connman_service_get_proxy_method(struct connman_service *service);
+char **connman_service_get_proxy_servers(struct connman_service *service);
+char **connman_service_get_proxy_excludes(struct connman_service *service);
+const char *connman_service_get_proxy_url(struct connman_service *service);
+const char *connman_service_get_proxy_autoconfig(struct connman_service *service);
 
 #ifdef __cplusplus
 }
