@@ -24,6 +24,8 @@
 #include <config.h>
 #endif
 
+#include <errno.h>
+
 #include <gdbus.h>
 
 #include "connman.h"
@@ -194,6 +196,8 @@ static enum connman_service_type bearer2service(const char *bearer)
 		return CONNMAN_SERVICE_TYPE_BLUETOOTH;
 	else if (g_strcmp0(bearer, "3g") == 0)
 		return CONNMAN_SERVICE_TYPE_CELLULAR;
+	else if (g_strcmp0(bearer, "vpn") == 0)
+		return CONNMAN_SERVICE_TYPE_VPN;
 	else
 		return CONNMAN_SERVICE_TYPE_UNKNOWN;
 }
@@ -211,10 +215,11 @@ static char *service2bearer(enum connman_service_type type)
 		return "bluetooth";
 	case CONNMAN_SERVICE_TYPE_CELLULAR:
 		return "3g";
+	case CONNMAN_SERVICE_TYPE_VPN:
+		return "vpn";
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
 	case CONNMAN_SERVICE_TYPE_SYSTEM:
 	case CONNMAN_SERVICE_TYPE_GPS:
-	case CONNMAN_SERVICE_TYPE_VPN:
 	case CONNMAN_SERVICE_TYPE_GADGET:
 		return "";
 	}
@@ -635,11 +640,12 @@ static gint sort_allowed_bearers(struct connman_service *service_a,
 
 static gint sort_services(gconstpointer a, gconstpointer b, gpointer user_data)
 {
-	struct connman_service *service_a = (void *)a;
-	struct connman_service *service_b = (void *)b;
+	struct service_entry *entry_a = (void *)a;
+	struct service_entry *entry_b = (void *)b;
 	struct connman_session *session = user_data;
 
-	return sort_allowed_bearers(service_a, service_b, session);
+	return sort_allowed_bearers(entry_a->service, entry_b->service,
+				session);
 }
 
 static void cleanup_session(gpointer user_data)
